package BDSM::Map::AStar;

use strict;
use warnings;
use include;

use BDSM::Vector;

# We extend BDSM::Map with even more methods. They don't modify the map, rather they give it
# metadata and provide a framework for collaborative emergent antiobject intelligence.
# Fancy, eh?
use BDSM::Map;
our @ISA = ("BDSM::Map");

# Go ahead and make the map. No special stuff with us.
sub new {
  return BDSM::Map::new(@_);
}

# Register an agent tracking another agent.
sub track {
  my ($self, $agent, $goal, $offset) = @_;
  $self->{Paths}{$agent} = {
    Goal   => $goal,
    Offset => $offset,
    Tick   => 0
  };
}

# Tell an agent the next step in pursuing its goal. Actually run the good ol' A*.
sub nextmove {
  my ($map, $src) = @_;

  my $path = $map->{Paths}{$src};
  my $y1 = $src->{Y};
  my $x1 = $src->{X};
  my $y2 = $path->{Goal}{Y} + $path->{Offset}[0];
  my $x2 = $path->{Goal}{X} + $path->{Offset}[1];

  return if $y1 == $y2 and $x1 == $x2;

  #debug "$src->{Symbol} from $y1, $x1 to $y2, $x2";

  my $open = Heap::Priority->new;
  $open->lowest_first;
  my $startpri = abs($y1 - $y2) + abs($x1 - $x2);
  $open->add({ Y => $y1, X => $x1, P => $startpri }, $startpri);

  # Make a temp copy of the map
  my $virtmap = [];
  foreach my $y (0 .. $map->height) {
    foreach my $x (0 .. $map->width) {
      $virtmap->[$y][$x] = {};
    }
  } 
  $virtmap->[$y1][$x1]{Used} = 1;

  my @back;
  while (my $node = $open->pop) {
    if ($node->{Y} == $y2 and $node->{X} == $x2) {
      # We're here!
      my $y = $node->{Y};
      my $x = $node->{X};
      my @path = ([$y, $x]);
      # From the destination node, trace back to Start.
      while ($virtmap->[$y][$x]{Backref}) {
        my $backref = $virtmap->[$y][$x]{Backref};
        ($y, $x) = @$backref;
        unshift @path, $backref;
      }
      return @{ $path[1] };
    }

    # Record this node as backup if it's the closest we might get
    my $manh = abs($y2 - $node->{Y}) + abs($x2 - $node->{X});
    if ($#back == -1 or $manh < $back[0]) {
      @back = ($manh, $node->{Y}, $node->{X});
    }

    foreach (adjacent_tiles("diag", $node->{Y}, $node->{X})) {
      my ($y, $x) = @$_;
      next if $virtmap->[$y][$x]{Used};   # Pointless trails, dude.
      next unless $map->{Map}[$y][$x]{_} eq "." or $map->{Map}[$y][$x]{_} eq $path->{Goal};

      my $h = abs($y - $y2) + abs($x - $x2);
      $virtmap->[$y][$x]{Used} = 1;
      $virtmap->[$y][$x]{Backref} = [ $node->{Y}, $node->{X} ];
      $open->add({
        P => $h, 
        Y => $y,
        X => $x
      });
    }
  }
  # Do what we can.
  return unless @back;
  # We're here!
  my $y = $back[1];
  my $x = $back[2];
  my @path = ([$y, $x]);
  # From the destination node, trace back to Start.
  while ($virtmap->[$y][$x]{Backref}) {
    my $backref = $virtmap->[$y][$x]{Backref};
    ($y, $x) = @$backref;
    unshift @path, $backref;
  }
  return @{ $path[1] } if $path[1];
}

42;
