package BDSM::Map::CollabDiffusion;

use strict;
use warnings;
use include;

use BDSM::Vector;

# We extend BDSM::Map with even more methods. They don't modify the map, rather they give it
# metadata and provide a framework for collaborative emergent antiobject intelligence.
# Fancy, eh?
use BDSM::Map;
our @ISA = ("BDSM::Map");

# Go ahead and make the map. No special stuff with us.
sub new {
  return BDSM::Map::new(@_);
}

# Register an agent tracking another agent.
sub track {
  my ($self, $agent, $goal, $offset) = @_;
  $self->{Paths}{$agent} = {
    Goal   => $goal,
    Offset => $offset,
    Tick   => 0
  };
}

# Tell an agent the next step in pursuing its goal.
sub nextmove {
  my ($self, $src) = @_;

  my $path = $self->{Paths}{$src};

  # Update the influence map.
  $self->_diffuse($src);

  my $ticks = $path->{Tick};

  my @best;
  foreach (adjacent_tiles("diag", $src->{Y}, $src->{X})) {
    my ($y, $x) = @$_;
    my $try = $self->tile($y, $x);
    next if $#{ $try->{Scents}{$src} } == -1;
    next unless $ticks == $try->{Scents}{$src}[1];
    if ($#best == -1 or $try->{Scents}{$src}[0] < $best[0]) {
      @best = ($try->{Scents}{$src}[0], $y, $x);
    }
  }
  if (@best) {
    return ($best[1], $best[2]);
  } else {
    return;
  }
}

# Recalculate the influence map.
sub _diffuse {
  my ($map, $src) = @_;

  my $path = $map->{Paths}{$src};
  my $toy = $path->{Goal}{Y} + $path->{Offset}[0];
  my $tox = $path->{Goal}{X} + $path->{Offset}[1];
  my $tick = ++$path->{Tick};

  my @nodes = ([$toy, $tox]);
  $map->tile($toy, $tox)->{Scents}{$src} = [0, $tick];
  my @back;
  my $cnt = 0;
  while (my $node = shift @nodes) {
    $cnt++;
    my $scent = $map->tile(@$node)->{Scents}{$src}[0] + 1;
    # Add adjacents to the heap and give them their scent level.
    foreach (adjacent_tiles("cardinal", @$node)) {
      my ($y, $x) = @$_;
      my $new = $map->tile($y, $x);
      # If we've already dealt with it this tick, ignore it.
      next if defined $new->{Scents}{$src}[1] and $new->{Scents}{$src}[1] == $tick;
      # Don't add it unless it's permeable.
      if ($new->{Actor}) {
        # Well, fine, but its a bad idea i tell ya.
        $new->{Scents}{$src} = [$scent + 1000, $tick];
        push @back, [$y, $x];
      } elsif ($new->{_} eq ".") {
        $new->{Scents}{$src} = [$scent, $tick];
        push @nodes, [$y, $x];
      }
    }
    unless ($nodes[0]) {
      last if $cnt == 1;
      push @nodes, @back;
      @back = ();
    }
  }
}

42;
