package BDSM::Map::Level;

use strict;
use warnings;
use include;

# We extend BDSM::Map by populating it with stuff.

# TODO: how to handle fixed areas?
# TODO: We're somehow called on a generated map.
sub addlevel {
  my $map = shift;

  # Add the level to the Game DB
  my $z = ++GAME->{LevelCnt};
  GAME->{Levels}{$z} = $map;
  $map->{Depth} = $z;

  populate($map);
  linkstairs($map, GAME->{Levels}{ GAME->{LevelCnt} - 1}) unless $z == 1;
}

# Link two maps' staircases to each other.
sub linkstairs {
  my ($lower, $upper) = @_;

  # Link the lower to the upper
  my $stair = 0;
  foreach (@{ $lower->{StairsUp} }) {
    $stair = 0 if $stair > $#{ $upper->{StairsDown} };  # Loop through available links
    $lower->{Map}[ $_->[0] ][ $_->[1] ]{Stair} = $upper->{StairsDown}[$stair++];
  }

  # And the upper to the lower
  $stair = 0;
  foreach (@{ $upper->{StairsDown} }) {
    $stair = 0 if $stair > $#{ $lower->{StairsUp} };  # Loop through available links
    $upper->{Map}[ $_->[0] ][ $_->[1] ]{Stair} = $lower->{StairsUp}[$stair++];
  }
}

# Populate a map with appropriate stairs, monsters, and items given its depth
sub populate {
  my $map = shift;

  # TODO: more even distribution could happen if we generate hotspots from within each room
  # and store those

  # Generate stairs. Fixed at random(2, 4) for now.
  $map->{StairsUp} = [];
  $map->{StairsDown} = [];
  foreach my $type ("StairsUp", "StairsDown") {
    foreach (1 .. random(2, 4)) {
      my ($y, $x) = $map->getpt(".");
      $map->{Map}[$y][$x]{_} = $type eq "StairsUp" ? "<" : ">";
      push @{ $map->{$type} }, [$y, $x];
    }
  }
}

42;
