package BDSM::Map::Transform;

use strict;
use warnings;
use include;

use BDSM::Vector;

# We simply extend BDSM::Map with more methods. Those in BDSM::Map are accessors mostly; we
# transform maps... Wow, you'd think package names would be obvious.

# Fills the map with the specified tile from range [Y, X] to [Y, X].
sub fill {
  my ($self, $from, $to, $with) = @_;
  # What happens in the dungeon stays in the dungeon...
  # (Old joke when I used to screw up encapsulation. STILL RELEVANT SOMEHOW.)
  $self->check(@$from);
  $self->check(@$to);
  my ($y1, $x1) = @$from;
  my ($y2, $x2) = @$to;
  foreach my $y ($y1 .. $y2) {
    foreach my $x ($x1 .. $x2) {
      $self->{Map}[$y][$x]{_} = $with;
    }
  }
}

# Properly border an entire map with some complex rules.
sub border {
  # TODO: Here be dragons... I've forgotten all of this. No really, this is
  # magick.
  my $self = shift;
  my $map = $self->{Map};
  my $cnty = 0;
	# We munge directly cause we can and cause it's faster.
  foreach my $y (0 .. $self->height) {
    my $cntx = 0;
    foreach my $x (0 .. $self->width) {
      $map->[$y][$x]{_}= "#"
      if
        $y != 0
      and
        $map->[$y - 1][$x]{_} eq "."
      and
        $map->[$y][$x]{_} eq " ";
      $map->[$y][$x]{_} = "#"
      if
        $y != 0
      and
        $x != 0
      and
        $map->[$y - 1][$x - 1]{_} eq "."
      and
        $map->[$y][$x]{_} eq " ";
      $map->[$y][$x]{_} = "#"
      if
        $y != 0
			and
				$x != $self->width
      and
        $map->[$y - 1][$x + 1]{_} eq "."
      and
        $map->[$y][$x]{_} eq " ";
      next unless $map->[$y][$x]{_} eq ".";
      $map->[$y][$x]{_} = "#" if $cnty == 0 or $cntx == 0;
      $map->[$y][$x]{_} = "#" if $y != 0 and $map->[$y - 1][$x]{_} eq " ";
      $map->[$y][$x]{_} = "#" if $x != 0 and $map->[$y][$x - 1]{_} eq " ";
      $map->[$y][$x]{_} = "#"
			if
				$x != $self->width
      and
        $map->[$y][$x + 1]{_} eq " ";
      $map->[$y][$x]{_} = "#"
      if
        $y != 0
      and
        $x != 0
      and
        $map->[$y - 1][$x - 1]{_} eq " ";
      $map->[$y][$x]{_} = "#"
      if
        $y != 0
			and
				$x != $self->width
      and
        $map->[$y - 1][$x + 1]{_} eq " ";
      $map->[$y][$x]{_} = "#" if $cnty == $self->height;
      $cntx++;
    }
    $map->[$y][-1]{_} = "#" if $map->[$y][-1]{_} eq ".";
    $cnty++;
  }
}

# Run game of life rules for a number of iterations.
sub cellular {
  my ($map, $iters) = @_;
  my $old = [];
  $iters++ if $iters % 2 == 0;  # Even number of iterations "inverts" the map
  foreach (1 .. $iters) {
    # Store old copy!
    foreach my $y (0 .. $map->height) {
      foreach my $x (0 .. $map->width) {
        $old->[$y][$x] = $map->{Map}[$y][$x]{_};
      }
    }

    # Apply Game of Life rules
    foreach my $y (1 .. $map->height - 1) {
      foreach my $x (1 .. $map->width - 1) {
        # How many neighbors?
        my $neighbors = 0;
        $neighbors++ if $old->[$y - 1][$x - 1] eq "#";
        $neighbors++ if $old->[$y - 1][$x]     eq "#";
        $neighbors++ if $old->[$y - 1][$x + 1] eq "#";
        $neighbors++ if $old->[$y][$x - 1]     eq "#";
        $neighbors++ if $old->[$y][$x + 1]     eq "#";
        $neighbors++ if $old->[$y + 1][$x - 1] eq "#";
        $neighbors++ if $old->[$y + 1][$x]     eq "#";
        $neighbors++ if $old->[$y + 1][$x + 1] eq "#";
        $map->{Map}[$y][$x]{_} = $neighbors < 3 ? "#" : ".";
      }
    }
  }
}

# Simply change any walls to floors unless a blank space is adjacent to them.
sub excavate {
  my $map = shift;
  foreach my $y (1 .. $map->height - 1) {
    WALL: foreach my $x (1 .. $map->width - 1) {
      next unless $map->{Map}[$y][$x]{_} eq "#";
      foreach (adjacent_tiles("diag", $y, $x)) {
        my ($Y, $X) = @$_;
        next WALL if $map->{Map}[$Y][$X]{_} eq " ";
      }
      $map->{Map}[$y][$x]{_} = ".";
    }
  }
}

42;
