package Cursed::Window;

use strict;
use warnings;
use include;

use Curses;

# Create and return a new prompt window.
sub init {
  my ($class, %dat) = @_;
  my $self = bless \%dat, $class;
	$self->{Subwindows} = [];
  $self->spawn;
  if (%{ $self->{Opts} }) {
    debug $self->{Opts};
    die "unhandled options!";
  }
  delete $self->{Opts};
  return $self;
}

# Simply refresh the window
sub draw {
  my $self = shift;

  # Delegate most of the work to them.
	if ($self->can("windraw")) {
		$self->windraw;
	} else {
    # Unless they don't have to do anything fancy.
    $self->{Win}->refresh;
  }

	# Now update any subwindows lying on top of us.
	foreach my $sub (@{ $self->{Subwindows} }) {
    # Thanks to Thomas Dickey for the touchwin magic.
    touchwin($sub->{Win});
    $sub->{Win}->refresh;
    $sub->draw;
	}

  # And move the cursor to its spot if its on.
  if (my $cursor = $self->ui->{Cursor}) {
    $cursor->[0]{Win}->move($cursor->[1], $cursor->[2]);
    $cursor->[0]{Win}->refresh;
  }
}

# Give a reference to the HexedUI singleton
sub ui {
  my $self = shift;
  return $self->{_HexedUI};
}

# These are nice cause they're "encapsulated"
sub height {
  my $self = shift;
  return $self->{Height};
}

sub width {
  my $self = shift;
  return $self->{Width};
}

# Unregister subwindows overlapping us.
sub remove_subwins {
  my ($self, $win) = @_;
  # Messy pattern. Modifying arrays in for loops is... owch.
  my @new;
  foreach (@{ $self->{Subwindows} }) {
    push @new, $_ unless $_->{Name} eq $win;
  }
  @{ $self->{Subwindows} } = @new;
}

# Register the cursor as being somewhere... until it's not.
sub cursor {
  my ($self, $y, $x) = @_;
  if (defined $y) {
    $self->ui->{Cursor} = [$self, $y, $x];
  } else {
    delete $self->ui->{Cursor};
  }
}

42;
