package EventDB;

use strict;
use warnings;
use include;

# Set up the DB.
GAME->{EventDB} = bless {}, "EventDB";
GAME->{Objects} = [];

# Install an event reaction.
sub react {
  my ($db, %dat) = @_;

  my $actor = $dat{actor}->id;
  my $stage = $dat{stage};
  my $verb = $dat{verb};
  my $role = $dat{role};
  my $reaction = $dat{by};

  my %extras;   # Undefined entries in the reaction bug me
  $extras{Override} = $dat{override} if $dat{override};
  $extras{ObjectType} = $dat{obj_isa} if $dat{obj_isa};
  $db->{$stage}{$verb}{$actor} ||= [];
  push @{ $db->{$stage}{$verb}{$actor} },
  {
    Actor    => $actor,
    Role     => $role,
    Reaction => $reaction,
    %extras
  };
}

# Send an event and call any reactions in the DB.
sub signal {
  my ($db, %args) = @_;
  my $stage = $args{when};
  my $verb = $args{action};
  my $origactor = $args{actor};
  my $heap = $args{heap};
  
  # Go through all of the bases like in g()
  my @bases = ($origactor, $origactor->bases);

  # Make a list of the first argument object's bases, if possible.
  my %isas;
  if ($heap->{Args}[0] and ref $heap->{Args}[0] eq "Object") {
    my $dirobj = $heap->{Args}[0];
    if (my @ls = $dirobj->bases) {
      foreach (@ls) {
        $isas{ $_->{Type} } = 1 if $_->{Type};
      }
    }
    # Also let individual objects be constraints.
    $isas{ $dirobj->id } = 1 unless $dirobj->{Type};
  }

  my @reaxns;
  my %override;   # Keys are the actors, values are the roles
  foreach my $actor (@bases) {
    next unless my $events = $db->{$stage}{$verb}{$actor->id};
    # Make note of anything we are to override
    foreach (@$events) {
      # First check to see if this reaction applies to us.
      next if $_->{ObjectType} and !$isas{ $_->{ObjectType} };

      push @reaxns, $_;
      next unless my $skip = $_->{Override};
      $override{ $skip->[0] }{ $skip->[1] } = 1;
    }
  }

  # Got the reactions, now execute them.
  foreach (@reaxns) {
    if ($override{ $_->{Actor} }{ $_->{Role} }) {
      log_this("WARNING: skipping reaction for $stage $verb bc role = $_->{Role}");
      next;
    }
    log_this($origactor->id . " is doing a reaxn. $_->{Actor}'s $stage $verb, role=$_->{Role}");
    my @code = $_->{Reaction}->($origactor, $heap);

    return @code if $code[0] and $code[0] =~ m/^:(STOP|REDIRECT)/;
    # Otherwise, fine.
  }

  return 1;   # If there were no reactions, then nothing to complain about.. keep going.
}

42;
