package Hexed::Msg;

use strict;
use warnings;
use include;

use constant max_pad_items => 10000;

# We pretty much subclass Menu completely, wrapping things.
use Hexed::Menu;
our @ISA = ("Hexed::Menu");

use Queue;
use Curses;

# Create and return a new menu window.
sub spawn {
  my $self = shift;

  $self->{Queue} = new Queue;
  $self->{Queue}->hook(\&queue_hook, Msg => $self);

  $self->{Pad} = newpad(max_pad_items, $self->{Width});
  $self->{Offset} = 0;
}

# Overloaded... Sort of. We're the method that actually prints to the screen. Through pads.
sub windraw {
  my $self = shift;

	$self->{Pad}->prefresh(
	  $self->{Offset}, 0,                                               # Offset in the pad
	  $self->{AbsPosY} + $self->{Y1}, $self->{AbsPosX} + $self->{X1},   # Where in window
	  # TODO: potential offset issue. why is -1 needed?
	  $self->{AbsPosY} + $self->{Y2} - 1, $self->{AbsPosX} + $self->{X2} - 1
	);
}

# Every time something happens to our queue, do something. Potentially. Yes.
sub queue_hook {
  my $queue = shift;
  my $win = $queue->{Heap}{Msg};
  my $cmd = shift;

  # Scroll if we need to.
  if ($queue->all <= $win->{Height}) {
    $win->{Offset} = 0;
  } else {
    $win->{Offset} = $queue->all - $win->{Height};
  }

  # Add the message to the pad here for MAJOR efficency boost.
  my $y = $queue->all - 1;
  $win->{Pad}->standend;
  $win->{Pad}->addstr($y, 0, $queue->get($y)->{Msg});

  $win->draw;
}

# Simply add a message to the console.
sub addmsg {
  my ($self, $msg) = @_;
  $self->{Queue}->add($#{ $self->{Queue}{Queue} }, Msg => $msg);
}

# Just draw normally.
sub redraw {
  my $self = shift;
  $self->draw;
}

# Scroll the message log up if we can
sub scroll_up {
  my $self = shift;
  if ($self->{Offset} != 0) {
    $self->{Offset}--;
    $self->draw;
  }
}

# Scroll the message log down if we can
sub scroll_down {
  my $self = shift;
  if ($self->{Queue}->all > $self->{Height} and $self->{Offset} != $self->{Queue}->all -
      $self->{Height})
  {
    $self->{Offset}++;
    $self->draw;
  }
}

42;
