package Hexed::Prompt;

use strict;
use warnings;
use include;

use Cursed::Window;
our @ISA = ("Cursed::Window");

use Curses;

# Create and return a new prompt window.
sub spawn {
  my $self = shift;
}

# Go through the entire prompting process
sub prompt {
  my ($self, $query, $limit) = @_;
  # TODO: messy debugging, handling, testing
  die "prompt query is too long!" if length($query) > $self->{Width};
  die "prompt limit is too long!" if $limit > $self->{Width};

  $self->ui->start_in;

  # TODO: messy... all of it! a pattern? Fixed's job somehow?
  # Set the prompt title.
  $self->{Win}->standend;
  $self->{Win}->attron(A_REVERSE);
  # Center the query.
  my $x = center($query, $self->{Width});
  $self->{Win}->addstr($self->{Y1}, $self->{X1} + $x, $query);
  $self->draw;

  $self->start_prompt($limit);
  my $key;
  $self->nextkey($key) until ($key = $self->ui->wait_input) eq "<^M>";

  my $input = $self->stop_prompt;
  $self->ui->stop_in;

  return $input;
}

# The beginning of the sequence. Turn on the cursor and reset our data.
sub start_prompt {
  my ($self, $limit) = @_;

  # Turn the cursor on and start the magic loop, maaan.
  $self->{Win}->standend;
  curs_set(1);
  $self->cursor($self->{Y1}, $self->{X1});
  $self->draw;

  $self->{In} = [];
  $self->{X} = 0;
  $self->{Limit} = $limit;
}

# If we don't want to take exclusive control of the keyboard, can be called manually.
# Otherwise the full prompting procedure uses us anyway.
sub nextkey {
  my ($self, $key) = @_;
  
  # Grab our vars
  my $x = $self->{X};
  my @in = @{ $self->{In} };

  if ($key eq "<KEY_LEFT>") {
    $x-- unless $x == 0;
  } elsif ($key eq "<KEY_RIGHT>") {
    $x++ unless $x == @in;  # We can move one past the next character
  } elsif ($key eq "<KEY_HOME>") {
    $x = 0;
  } elsif ($key eq "<KEY_END>") {
    $x = @in;
  } elsif ($key eq "<KEY_BACKSPACE>") {
    unless ($x == 0) {
      splice @in, $x - 1, 1;
      $x--;
      $self->cursor($self->{Y1}, $self->{X1} + $x);
      $self->{Win}->clrtoeol;
      # A full redraw is quite unnecessary
      $self->{Win}->box(0, 0) if $self->{Border};
    }
  } elsif ($key eq "<KEY_DC>") {
    return if $x == @in;
    splice @in, $x, 1;
    $self->{Win}->clrtoeol;
    $self->{Win}->box(0, 0) if $self->{Border};
  } elsif ($key =~ m/<.+>/) {
    # Don't do anything with it
  } else {
    unless (@in == $self->{Limit}) {
      splice @in, $x, 0, $key;
      $x++;
    }
  }
  $self->{Win}->addstr($self->{Y1}, $self->{X1}, join("", @in));
  $self->cursor($self->{Y1}, $self->{X1} + $x);
  $self->draw;

  # Store our vars
  $self->{X} = $x;
  $self->{In} = \@in;
}

sub stop_prompt {
  my $self = shift;

  my $input = join "", @{ $self->{In} };
  delete $self->{X};
  delete $self->{Limit};
  delete $self->{In};

  # And clean up the window
  curs_set(0);
  $self->{Win}->move($self->{Y1}, $self->{X1});
  $self->{Win}->clrtoeol;
  $self->draw;
  return $input;
}

42;
