package StdLib::Character;

use strict;
use warnings;
use include;

use BDSM::DunGen::Dungeon;  # Because we want to create them sometimes
use Container;

use StdLib;

my $char = create("Character", From => "Agent");

$char->{Init} = sub {
  my ($self, $opts) = @_;

  $self->{Inv} = Container->new($self);

  # Equip our shit
  $self->{Equipment} = {};
  $self->equip($opts->{Wielding}, "Weapon") if $opts->{Wielding};

  if ($opts->{Wearing}) {
    foreach (@{ $opts->{Wearing} }) {
      # TODO: whip out the Pre logic to autoselect slot?
      $self->equip(@$_);
    }
  }

  # TODO: Schedule our stat managers

  return "Agent";  # Run their fun stuff
};

### methods

# Describe ourselves
$char->{Methods}{name} = sub {
  my $self = shift;
  my $name = $self->g("Name");

  # modifiers.. angry, berserk?

  die "unnamed character!" unless $name;
  return $name;
};

# Do we have the object in our inventory?
$char->{Methods}{has} = sub {
  my ($self, $item) = @_;
  return $item->{Owner} == $self->{Inv};
};

### idle

$char->react("On", "idle",
  role => "do",
  by   => sub {
    # Do... nothing?
  }
);

### take


# This is an exception... assumes all arguments are Items. Multi-arg; that's why we don't
# use obj_isa.
$char->react("On", "take",
  role => "do",
  by   => sub {
    my ($self, $heap) = @_;
    my (@items) = @{ $heap->{Args} };

    my $cnt = 0;
    foreach my $item (@items) {
      $item->{Owner}->del($item);     # Get off the tile
      $self->{Inv}->add($item);       # And into their grubby hands
      $heap->{Args}[$cnt++] = $item;  # THe container may have modified it
    }
  }
);

$char->react("After", "take",
  role => "msg",
  by   => sub {
    my ($self, $heap) = @_;
    my (@items) = @{ $heap->{Args} };

    $self->msg("[subj] [pick] up [1].", $_) foreach @items;
  }
);

### drop

# TODO: Again, how could we drop anything but an Item? No need for obj_isa
$char->react("Before", "drop",
  role => "check",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    if (!$self->has($item)) {
      return STOP($self, "[subj] [don't/doesn't] have that!");
    }

    if ($item->{On}) {
      # It's equipped; so try to unequip it first.
      return STOP unless $self->try_unequip($item);
    }
  }
);

$char->react("On", "drop",
  role => "do",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    $item->{Owner}->del($item);                                  # Get out of their hands
    $self->{Map}->inv($self->{Y}, $self->{X}, "add", $item);     # And into onto the tile
  }
);

$char->react("After", "drop",
  role => "msg",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    $self->msg("[subj] [drop] [1].", $item);
  }
);

### equip

$char->react("Before", "equip",
  role => "check",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item, $slot) = @{ $heap->{Args} };

    return STOP($self, "[subj] can't equip [1]!", $item);
  }
);

$char->react("Before", "equip",
  role     => "check-obj",
  obj_isa  => "Equipment",
  override => ["Character" => "check"],
  by       => sub {
    my ($self, $heap) = @_;
    my ($item, $slot) = @{ $heap->{Args} };

    # Do we have the item?
    return STOP($self, "[subj] don't [have/has] [1]!", $item) unless $self->has($item);

    # Is the item already equipped?
    return STOP($self, "[subj] already [have/has] [1] equipped!", $item) if $item->{On};

    # Does the item fit there?
    return STOP($self, "[subj] can't [equip] [1] there!", $item) unless grep(/$slot/, @{ $item->g("Fits") });

    # Is that slot free? Try to unequip it if we do.
    if ($self->{Equipment}{$slot}) {
      return STOP unless $self->try_unequip($self->{Equipment}{$slot});
    }
  }
);

$char->react("On", "equip",
  role => "do",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item, $slot) = @{ $heap->{Args} };

    $self->{Equipment}{$slot} = $item;
    $item->{On} = $slot;
  }
);

$char->react("After", "equip",
  role => "msg",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item, $slot) = @{ $heap->{Args} };

    # TODO: remove the (worn/wielded) modifier
    my $verb = $slot eq "Weapon" ? "[wield]" : "[put] on";
    $self->msg("[subj] $verb [1].", $item);
  }
);

### unequip

$char->react("Before", "unequip",
  role => "check",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item, $slot) = @{ $heap->{Args} };

    return STOP($self, "[subj] can't unequip [1]!", $item);
  }
);

$char->react("Before", "unequip",
  role     => "check-obj",
  obj_isa  => "Equipment",
  override => ["Character" => "check"],
  by       => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    # Do we have it?
    return STOP($self, "[subj] don't [have/has] [1]!", $item) unless $self->has($item);

    # Is it equipped?
    return STOP($self, "[subj] [aren't/isn't] equipping [1]!", $item) unless $item->{On};
  }
);

$char->react("On", "unequip",
  role => "do",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    delete $self->{Equipment}{ $item->{On} };
    $heap->{OldSlot} = delete $item->{On};  # Good to know for messaging
  }
);

$char->react("After", "unequip",
  role => "msg",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    my $verb = $heap->{OldSlot} eq "Weapon" ? "[unwield]" : "[take] off";
    $self->msg("[subj] $verb [1].", $item);
  }
);

### eat

$char->react("Before", "eat",
  role => "check",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item, $slot) = @{ $heap->{Args} };

    return STOP($self, "[subj] can't eat [1]!", $item);
  }
);

$char->react("Before", "eat",
  role     => "check-obj",
  obj_isa  => "Food",
  override => ["Character" => "check"],
  by       => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    # Do we have the item?
    return STOP($self, "[subj] don't [have/has] [1]!", $item) unless $self->has($item);
  }
);

$char->react("On", "eat",
  role => "do",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    # The item must provide the effect; we merely diminish its quantity.
    $item->destroy;
  }
);

$char->react("After", "eat",
  role => "msg",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    $self->msg("[subj] [eat] [1].", $item);
  }
);

### quaff

$char->react("Before", "quaff",
  role => "check",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item, $slot) = @{ $heap->{Args} };

    return STOP($self, "[subj] can't quaff [1]!", $item);
  }
);

$char->react("Before", "quaff",
  role     => "check-obj",
  obj_isa  => "Potion",
  override => ["Character" => "check"],
  by       => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    # Do we have the item?
    return STOP($self, "[subj] don't [have/has] [1]!", $item) unless $self->has($item);
  }
);

$char->react("On", "quaff",
  role => "do",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    # The item must provide the effect; we merely diminish its quantity.
    $item->destroy;
  }
);

$char->react("After", "quaff",
  role => "msg",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    $self->msg("[subj] [quaff] [1].", $item);
  }
);

### read

$char->react("Before", "read",
  role => "check",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item, $slot) = @{ $heap->{Args} };

    return STOP($self, "[subj] can't read [1]!", $item);
  }
);

$char->react("Before", "read",
  role     => "check-obj",
  obj_isa  => "Scroll",
  override => ["Character" => "check"],
  by       => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    # Do we have the item?
    return STOP($self, "[subj] don't [have/has] [1]!", $item) unless $self->has($item);
  }
);

$char->react("On", "read",
  role => "do",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    # The item must provide the effect; we merely diminish its quantity.
    $item->destroy;
  }
);

$char->react("After", "read",
  role => "msg",
  by   => sub {
    my ($self, $heap) = @_;
    my ($item) = @{ $heap->{Args} };

    $self->msg("[subj] [read] [1].", $item);
  }
);

### changedepth

$char->react("Before", "changedepth",
  role => "check",
  by   => sub {
    my ($self, $heap) = @_;
    my ($dir) = @{ $heap->{Args} };

    my $tile = $self->tile->{_};
    return STOP($self, "[subj] can't go up here!") if $dir eq "up" and $tile ne "<";
    return STOP($self, "[subj] can't go down here!") if $dir eq "down" and $tile ne ">";

    # Is there an available level?
    my $z = $self->{Map}{Depth};
    $z++ if $dir eq "down";
    $z-- if $dir eq "up";
    $heap->{Z} = $z;
    unless (GAME->{Levels}{$z}) {
      return STOP("You can't leave the dungeon... yet.") if $dir eq "up";
      # Generate a new dungeon if we're going deeper in.
      my $new = BDSM::DunGen::Dungeon->generate(50, 100);
      $new->addlevel;
    }
  }
);

$char->react("On", "changedepth",
  role => "do",
  by   => sub {
    my ($self, $heap) = @_;
    my ($dir) = @{ $heap->{Args} };
    my $z = $heap->{Z};

    # Calculate where we're going.
    my $newmap = GAME->{Levels}{$z};
    my ($y, $x) = @{ $self->tile->{Stair} };

    # TODO: messy go()ing!
    $self->_blitoff;
    $self->{Map} = $newmap;
    ($self->{Y}, $self->{X}) = ($y, $x);
    $self->_bliton;
    # TODO: blocking stairs

    $self->msg("[subj] [ascend] a level.") if $dir eq "up";
    $self->msg("[subj] [descend] a level.") if $dir eq "down";
  }
);

42;
