package StdLib::Character::Player;

use strict;
use warnings;
use include;

use StdLib;

my $player = create("Player", From => "Character",
  UniqueName       => "you",
  UniquePossessive => "your",
  Stages           => ["Pre", "Before", "On", "After"]  # Allow UI-user interaction
);

# Describe the contents of the tile.
$player->{Methods}{lstile} = sub {
  my $self = shift;
  my $implicit = shift;   # Did the user explicitly invoke us?

  if ($self->tile->{_} eq "<") {
    msg "There is a staircase leading up.";
  } elsif ($self->tile->{_} eq ">") {
    msg "There is a staircase leading down.";
  }
  my @ls = $self->{Map}->inv($self->{Y}, $self->{X}, "all");
  if (@ls == 1) {

    $self->msg("[subj] [see] here " . $ls[0]->general_name . ".");
  } elsif (@ls > 1) {
    $self->msg("[subj] [see] here:");
    foreach (@ls) {
      msg " " . $_->general_name;
    }
  } else {
    msg "There's nothing here." unless $implicit;
  }
};

# Display our inventory.
$player->{Methods}{inv} = sub {
  my ($self, @cats) = @_;
  if (@cats) {
    # Make sure the categories specified have something in them.
    @cats = grep { $self->{Inv}{Category}{$_} } @cats;
  } else {
    @cats = (keys %{ $self->{Inv}{Category} });
  }

  my @lines = ("Inventory");

  foreach my $cat (@cats) {
    push @lines, $cat;
    foreach my $item ($self->{Inv}->all($cat)) {
      push @lines, " $item->{Index} - " . $item->general_name;
    }
  }
  if (@lines) {
    return GAME->{UI}->popup(@lines);
  } else {
    $self->msg("Sorry, [subj] [don't/doesn't] HAVE anything!");
    return;
  }
};

# Choose an item from our inventory.
$player->{Methods}{selectinv} = sub {
  my ($self, $query, $categories, $otherkeys) = @_;
  my %otherkeys = map { $_ => 1 } @$otherkeys;

  GAME->{UI}->start_in;

  my $return;
  while (1) {
    msg $query;
    my $in = GAME->{UI}->wait_input;
    if ($in eq "<ESCAPE>" or $in eq " ") {
      msg "Never mind.";
      last;
    } elsif ($in eq "?") {
      $in = $self->inv(@$categories);
      last unless $in;  # We don't have anything. ^^
    } elsif ($otherkeys{$in}) {
      $return = $in;
      last;
    }
    if (my $item = $self->{Inv}->get($in)) {
      $return = $item;
      last;
    } else {
      $self->msg("[subj] [don't/doesn't] have that.");
    }
  }
  GAME->{UI}->stop_in;
  return $return;
};

$player->{Methods}{test} = sub {
  my $self = shift;
  debug GAME;
};

### go

$player->react("After", "go",
  role => "msg",
  by   => sub {
    my ($self, $heap) = @_;

    $self->lstile("quiet");
  }
);

### take

$player->react("Pre", "take",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    my @ls = $self->{Map}->inv($self->{Y}, $self->{X}, "all");
    if (@ls == 0) {
      return STOP("There's nothing here!");
    } elsif (@ls == 1) {
      $heap->{Args} = \@ls;
    } else {
      GAME->{UI}->start_in;
      msg "There are " . scalar @ls . " items here:";
      my $all = 0;  # Get the rest?
      my @get;
      foreach my $obj (@ls) {
        push(@get, $obj), next if $all;
        $obj->msg("Pick up [subj]? (y,n,a,q)");
        my $in = GAME->{UI}->wait_input;
        last if $in eq "q";
        @get = (), last if $in eq " " or $in eq "<ESCAPE>";
        next if $in eq "n";
        $all = 1 if $in eq "a";
        push @get, $obj;
      }
      GAME->{UI}->stop_in;
      if (@get) {
        $heap->{Args} = \@get;
      } else {
        return STOP("None of it was worth it anyway.");
      }
    }
  }
);

$player->react("After", "take",
  role     => "msg",
  override => ["Character" => "msg"],
  by       => sub {
    my ($self, $heap) = @_;
    my (@items) = @{ $heap->{Args} };

    msg "$_->{Index} - " . $_->general_name foreach @items;
  }
);

### drop

$player->react("Pre", "drop",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    return STOP unless my $item = $self->selectinv("Drop what? (? to list)");
    $heap->{Args} = [$item];
  }
);

### wield

$player->react("Pre", "wield",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    return STOP unless my $item = $self->selectinv("Wield what? (- for none)", ["Weapons"], ["-"]);
    if ($item eq "-") {
      if ($item = $self->{Equipment}{Weapon}) {
        return REDIRECT("unequip", $item);
      } else {
        return STOP($self, "[subj] [aren't/isn't] wielding anything!");
      }
    }
    return REDIRECT("equip", $item, "Weapon");
  }
);

### quickwield

$player->react("Pre", "quickwield",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    my @weapons = $self->{Inv}->all("Weapons");
    return STOP($self, "[subj] [have/has] no weapons!") unless @weapons;
    
    my $wieldme;
    if (my $current = $self->{Equipment}{Weapon}) {
      # Already got a weapon, so find the first one in here that isn't us
      foreach (@weapons) {
        next if $_->id eq $current->id;
        $wieldme = $_;
        last;
      }
      return STOP($self, "[subj] [have/has] no other weapons to wield!") unless $wieldme;
    } else {
      $wieldme = shift @weapons;
    }
    return REDIRECT("equip", $wieldme, "Weapon");
  }
);

### wear

$player->react("Pre", "wear",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    return STOP unless my $item = $self->selectinv("Wear what?", ["Armour"]);

    # It's pointless to find a slot if we're already wearing it. So steal that check from
    # Character's BEFORE_equip.
    return STOP($self, "[subj] [are/is] already wearing [1]!", $item) if $item->{On};

    # So find a slot for it.
    my @fits = @{ $item->g("Fits") };
    die "item has no slots at all defined" unless @fits;
    my (@free, @filled);
    foreach (@fits) {
      $self->{Equipment}{$_} ? push(@filled, $_) : push(@free, $_);
    }

    # If we have 1 free slot, fill it automatically.
    # But otherwise, let em choose from all the slots.
    my $slot;
    if (@free == 1) {
      $slot = shift @free;
    } else {
      my @slots;
      push @slots, "$_ (available)" foreach @free;
      push @slots, "$_ (filled)" foreach @filled;
      $slot = GAME->{UI}->choose(
        0,"Where do you want to wear " . $item->specific_name . "?", @slots
      );
      $slot = $slots[$slot];
      $slot =~ s/ \(.+$//;
    }
    return REDIRECT("equip", $item, $slot);
  }
);

### takeoff

$player->react("Pre", "takeoff",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    return STOP unless my $item = $self->selectinv("Take what off?", ["Armour"]);
    return REDIRECT("unequip", $item);
  }
);

### eat

$player->react("Pre", "eat",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    return STOP unless my $item = $self->selectinv("Eat what?", ["Food"]);
    $heap->{Args} = [$item];
  }
);

### quaff

$player->react("Pre", "quaff",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    return STOP unless my $item = $self->selectinv("Drink what?", ["Potions"]);
    $heap->{Args} = [$item];
  }
);

### read

$player->react("Pre", "read",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    return STOP unless my $item = $self->selectinv("Read what?", ["Scrolls"]);
    $heap->{Args} = [$item];
  }
);

### ascend

$player->react("Pre", "ascend",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    return REDIRECT("changedepth", "up",);
  }
);

### descend

$player->react("Pre", "descend",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    return REDIRECT("changedepth", "down");
  }
);

### changedepth

$player->react("After", "changedepth",
  role => "ui",
  by   => sub {
    my ($self, $heap) = @_;

    # We now have a new map to display.
    GAME->{UI}{Main}->bindmap($self->{Map});
  }
);

# TODO: examine (target)

42;
