package StdLib::Tilemap;

use strict;
use warnings;
use include;

# Provide our singleton.
GAME->{Tilemap} = bless {
  '#' => "orange",  # Wall
  '.' => "grey",    # Ground
  '<' => "white",   # Upstairs
  '>' => "white",   # Downstairs
}, __PACKAGE__;

# Determine the symbol and color of a particular tile.
sub render {
  my ($self, $tile) = @_;
  my ($symbol, $color);

  if ($tile->{Actor}) {
    # Is there an actor on here?
    $tile = $tile->{Actor};
    if (ref $tile eq "HASH") {
      # Probably some split-up flyweightedish sprite, grab directly.
      $symbol = $tile->{Symbol};
      $color = $tile->{Color};
    } else {
      # Some Object, handle it.
      $symbol = $tile->g("Symbol");
      $color = $tile->g("Color");
    }
  } elsif ($tile->{Inv} and my ($item) = $tile->{Inv}->all) {
    # Fine; an item?
    $symbol = $item->g("Symbol");
    $color = $item->g("Color");
  } else {
    # Alright, normal tile.
    $symbol = $tile->{_};
    $color = $tile->{Color};
    $color ||= $self->{$symbol};
  }
  return ($symbol, $color);
}

42;
