#!/usr/bin/perl

use strict;
use warnings;
use lib ".";
use include;

use Time::HiRes ("time");
my $start = time();
my $replicate = "";

my ($algo, $level) = @ARGV;
$algo ||= "block";
$level ||= "pacman1";

# & collab & astar..
use BDSM::Map::Blockmap;
use BDSM::Map::AStar;
use BDSM::Map::CollabDiffusion;

# Set up the map.
my $map;
if ($algo eq "astar") {
  $map = BDSM::Map::AStar->new("maps/$level");
} elsif ($algo eq "collab") {                                             
  $map = BDSM::Map::CollabDiffusion->new("maps/$level");                  
} elsif ($algo eq "block") {
  $map = BDSM::Map::Blockmap->new("maps/$level");
}
$map->{Moves} = 0;

# Load the templates, in order.
use BDSM::Agent;
use BDSM::Agent::Sprite;
use BDSM::Agent::Blob;
use StdLib::Character;
use StdLib::Character::Player;
use StdLib::Item;
use StdLib::Item::Equipment;
use StdLib::Item::SingleUse;

# Set up the agents
my $player = delete $map->{LoadedAgents}{"@"};
# This is cheating only a little
$player->{Base} = GAME->{Templates}{Player};
$player->{Color} = "Red";

### IMPORTANT. dont allow pushing!
GAME->{Templates}{Agent}{Methods}{pushedby} = sub { return; };

my @enemies = values %{ $map->{LoadedAgents} };
delete $map->{LoadedAgents};

# Set offsets and stuff
my $middle = int $#enemies / 2;
for (0 .. $middle) {
  my $agent = $enemies[$_];
  my $track;
  if ($_ == 0) {
    $track = $player;
  } else {
    $track = $enemies[$_ - 1];
  }
  $agent->{Map}->track($agent, $track, [+1, -1]);
}
for ($middle .. $#enemies) {
  my $agent = $enemies[$_];
  my $track;
  if ($_ == $middle) {
    $track = $player;
  } else {
    $track = $enemies[$_ - 1];
  }
  $agent->{Map}->track($agent, $track, [+1, +1]);
}

use HexedUI;

# Set up the interface.
my $ui = cast HexedUI {
  Main => {
    Type   => "Map",
    At     => [0, 0],
    Size   => ["50% + 3", "100%"],
    Border => 1,
    Pad    => 1,
    Map    => $map,
  },
  Msgs => {
    Type   => "Msg",
    At     => ["50% + 3", "65%"],
    Size   => ["50% - 3", "35% + 1"],
    Border => 1
  },
  Overview => {
    Type   => "Map",
    At     => ["50% + 3", 0],
    Size   => ["50% - 3", "65%"],
    Border => 1
  }
};
GAME->{UI} = $ui;
$ui->install_keymap({Name => "uhh yeah no"} => \&gametick);
msg(set => $ui->{Msgs});
$ui->{Overview}->bindmap($map->{Heap}{Blockmap}{Blockmap});
$ui->{Overview}->draw;

$ui->{Main}->focus($player);
$ui->{Main}->draw;

sub gametick {
  my (undef, $key) = @_;

  my $do = {
    h => "w",
    j => "s",
    k => "n",
    l => "e",
    y => "nw",
    u => "ne",
    b => "sw",
    n => "se",
  '.' => "idle",
  }->{$key};

  $player->$do if $do;

  my $erase;
  $ui->{Overview}->target({ Y => 0, X => 0 }, sub {

    my ($minimap, $Y, $X) = @_;
    if ($erase) {
      foreach my $y ($erase->{Y1} .. $erase->{Y2}) {
        foreach my $x ($erase->{X1} .. $erase->{X2}) {
          $ui->{Main}->_unhighlight($y, $x);
        }
      }
    }
    my $block = $minimap->tile($Y, $X)->{Room};
    msg "($Y, $X) is " . (abs($block->{Y1} - $block->{Y2}) * abs($block->{X1} - $block->{X2}));
    foreach my $y ($block->{Y1} .. $block->{Y2}) {
      foreach my $x ($block->{X1} .. $block->{X2}) {
        $ui->{Main}->_highlight($y, $x);
      }
    }
    $erase = $block;
  }) if $key eq "X";
  if ($erase) {
    foreach my $y ($erase->{Y1} .. $erase->{Y2}) {
      foreach my $x ($erase->{X1} .. $erase->{X2}) {
        $ui->{Main}->_unhighlight($y, $x);
      }
    }
  }

  $replicate .= $key;

  foreach (@enemies) {
    $_->go( $map->nextmove($_) );
  }

  $ui->{Main}->focus($player);
  $ui->{Main}->draw;
}

$ui->start;
exit;

END {
  debug "Moves: $map->{Moves}";
  debug sprintf("Time: %.1f", time() - $start);
  debug "To replicate: $replicate";
}
