package include;

use strict;
use warnings;

use Exporter;
our @ISA = ("Exporter");
our @EXPORT = qw(
  GAME debug center longest random percent choosernd msg log_this log_push log_pop
);

use Data::Dumper;
use feature "state";

# Since everything in PerlRL includes us, makes sense for us to do the work of sharing
# variables.
our $share = {};

# These are convenient interfaces
sub GAME {
  return $share;
}

# Data::Dumpers everything it gets to STDERR.
sub debug {
  print STDERR Dumper(@_);
}

# Provide the offset to center something.
sub center {
  my ($thing, $max) = @_;
  $thing = length($thing) unless $thing =~ m/^\d+$/;
  return int($max / 2) - int($thing / 2);
}

# Return the length of the longest member
sub longest {
  my $len = 0;
  foreach (@_) {
    $len = length($_) if length($_) > $len;
  }
  return $len;
}

# Return a random integer between bounds.
sub random {
  if (@_ == 1) {
    my $max = shift;
    return int rand($max + 1);
  } else {
    my ($min, $max) = @_;
    return $min if $min == $max;
    ($min, $max) = ($max, $min) if $min > $max;
    return $min + int rand($max - $min + 1);
  }
}

# Return true a percentage of the time.
sub percent {
  return 1 if random(100) <= shift;
}

# Choose something in a list randomly.
sub choosernd {
  return $_[ random(0, $#_) ];
}

# Do something arbitrary with a message from anywhere.
sub msg {
  state $handler;
  if (@_ == 2) {
    # Merely pass in an object with an addmsg() routine conforming to a simple declaration!
    $handler = $_[1];
  } else {
    if ($handler) {
      $handler->addmsg(shift);
    } else {
      debug "No msg handler yet, sorry";
    }
  }
}

my $logindent = 0;

# Record a message 
sub log_this {
  my $msg = shift || "";
  print STDERR " " x ($logindent * 2) . "$msg\n";
}

sub log_push {
  $logindent++;
  log_this(@_);
}

sub log_pop {
  $logindent--;
  log_this(@_);
}

42;
